################################################################################
#
# Module GeoFile::AmberCrd
#
# Author: Jan Rezac
# Date created: 2009-06-25
# License: Cuby license
# Description: AMBER .crd file writer
# Status: Working
#
################################################################################

module GeoFile
module AmberCrd
	def AmberCrd.write(geometry, arguments = {})
		arguments[:file] = $stdout if arguments[:file].nil?

		file = arguments[:file]

		close = false
		if file.class == String
			close = true
			mode="w+"
			file = File.new(file,mode)
		end
		unless file.kind_of?(IO)
			raise(TypeError,"File must be IO object")
		end

		file.printf("\n%6d\n",geometry.size)
		eo = false
		geometry.each { |atom| 
			file.printf("%12.7f%12.7f%12.7f",atom.x,atom.y,atom.z)
			file.printf("\n") if eo 
			eo = !eo
		}
		file.printf("\n") if eo 
		file.close if close
		return nil
	end

	def AmberCrd.read_into_geo(geometry, arguments = {})

		# Check file argument, open the file
		unless arguments[:file]
			raise(ArgumentError, "AmberCrd.read_into_geo requires argument :file")
		end
		file = arguments[:file]
		close = false
		if file.class == String
			close = true
			mode="r"
			file = File.new(file,mode)
		end
		unless file.kind_of?(IO)
			raise(TypeError,"File must be IO object")
		end

		# Read first empty line
		file.gets

		# Read number of atoms
		natom = file.gets.strip.to_i
		if natom != geometry.size
			raise("Number of atoms in AMBER crd file does not match number of atoms in geometry")
		end

		# Read all coordinates
		vec = []
		while line = file.gets
			line.strip.split.each{|x| vec << x.to_f if vec.size < natom * 3}
			break if vec.size >= natom * 3 # Do not read velocities
		end

		# Update the geometry
		geometry.update_from_vector!(vec)

		# Close the file
		file.close if close

		return nil
	end
end
end
