################################################################################
#
# Module GeoFile::DftbGen
#
# Author: Jan Rezac
# Date created: 2009-06-25
# License: Cuby license
# Description: Turbomole coord file reader/writer
# Status: Working
#
################################################################################

module GeoFile
module DftbGen

	def DftbGen.write(geometry, arguments = {})
		arguments[:file] = $stdout if arguments[:file].nil?
		file = arguments[:file]

		element_case = :Xy
		element_case = arguments[:element_case] if arguments[:element_case]

		close = false
		if file.class == String
			close = true
			mode="w+"
			file = File.new(file,mode)
		end
		unless file.kind_of?(IO)
			raise(TypeError,"File must be IO object")
		end

		# periodicity
		# default is cluster
		periodic = "C"
		if arguments[:periodic] 
			case arguments[:periodic]
			when :supercell
				periodic = "S"
			else
				raise "Gen format periodicity mode not accepted"
			end
		end

		# Output starts here

		# get elements in structure - array of atoms
		presentatoms = geometry.elements_in_system

		# write file
		file.puts "#{geometry.size}  #{periodic}"
		presentatoms.each { |a|
			element_string = a.to_s
			case element_case
			when :Xy
				# Do nothing
			when :XY
				element_string.upcase!
			when :xy
				element_string.downcase!
			else
				raise(ArgumentError, "Value of element_case must be :Xy, :XY or :xy")
			end
			file.printf("%s ", element_string)
		}
		file.puts
		geometry.each_index { |i|
			file.printf("%d %d %f %f %f\n",i+1,presentatoms.index(geometry[i].element)+1,geometry[i].x,geometry[i].y,geometry[i].z)
		}

		# Periodic box info
		if periodic == "S"
			# Origin coordinates (ignored by DFTB+)
			file.printf("%f %f %f\n", 0, 0, 0)
			# Lattice vectors
			arguments[:lattice].each{|vec|
				file.printf("%f %f %f\n", vec.x, vec.y, vec.z)
			}
		end

		# Output ends here

		file.close if close
		return nil
	end
end
end
