class TesselateTriangles
	attr_reader :triangles

	class Triangle
		# Coordinates of the three points
		attr_reader :a, :b, :c

		def initialize(a,b,c)
			@a = a
			@b = b
			@c = c
		end

		def center
			return (@a+@b+@c)/3.0
		end

		def each_point
			yield @a
			yield @b
			yield @c
		end

		def to_a
			return [@a,@b,@c]
		end

		def area
			return ((b-a).cross_product(c-a)).abs * 0.5
		end

		def translate!(coord)
			@a += coord
			@b += coord
			@c += coord
			return nil
		end
	end

	#=======================================================================
	# Initialization
	#=======================================================================
	
	def initialize(origin, r, n)
		# Radius, order (1 for icosahedron)
		@origin = origin
		@r = r
		@n = n

		raise "n must be at leat 1" if @n < 1
		
		# generate icosahedron
		@triangles = []
		apex = TesselateTriangles.sph2xyz(0.0, 0.0, @r)
		apex_l = -apex
		period = Math::PI * 2.0/5.0
		atan = Math::PI/2 - Math::atan(0.5)
		5.times{|i|
	 		# 5 triangles around upper apex
			a = TesselateTriangles.sph2xyz(atan, period * i, @r)
			b = TesselateTriangles.sph2xyz(atan, period * (i+1), @r)
			@triangles << Triangle.new(apex, a, b)
			# 5 triangles around lower apex
			c = TesselateTriangles.sph2xyz(atan, period * (i+0.5), @r)
			c.z = -c.z
			d = TesselateTriangles.sph2xyz(atan, period * (i+1.5), @r)
			d.z = -d.z
			@triangles << Triangle.new(apex_l, c, d)
			# 10 belt triangles
			@triangles << Triangle.new(a,b,c)
			@triangles << Triangle.new(b,c,d)
		}

		(n-1).times{
			new_triangles = []
			@triangles.each{|tri|
				a,b,c = tri.to_a
				# edge centers
				ab = (a+b)/2
				bc = (b+c)/2
				ca = (c+a)/2
				# move edge centers to @r
				ab = ab * @r/ab.abs
				bc = bc * @r/bc.abs
				ca = ca * @r/ca.abs
				# Four new triangles
				new_triangles << Triangle.new(a,ab,ca)
				new_triangles << Triangle.new(b,bc,ab)
				new_triangles << Triangle.new(c,ca,bc)
				new_triangles << Triangle.new(ab,bc,ca)
			}
			@triangles = new_triangles
		}

		# Add origin
		@triangles.each{|tri|
			tri.translate!(@origin)
		}
	end

	def iterate_centers
		@triangles.each{|tri|
			yield(tri.center)
		}
	end

	#=======================================================================
	# Helpers
	#=======================================================================

	def TesselateTriangles.sph2xyz(theta, phi, r)
		x = r * Math.sin(theta) * Math.cos(phi)
		y = r * Math.sin(theta) * Math.sin(phi)
		z = r * Math.cos(theta)
		return Coordinate[x,y,z]
	end
end

