module PredictiveIndex
	#: Calculation of Predictive index
	#: as defined in https://pubs.acs.org/doi/full/10.1021/jm0100279

	def PredictiveIndex.calc(x,y)
		n = x.size
		w = []
		c = []

		# get w and c
		n.times{|i|
			w[i] = []
			c[i] = []
			i.times{|j|
				w[i][j] = (x[j] - x[i]).abs
				c[i][j] = 0.0
				if y[j] != y[i]
					if (x[j]-x[i])/(y[j]-y[i]) < 0.0
						c[i][j] = -1.0 
					end
					if (x[j]-x[i])/(y[j]-y[i]) > 0.0
						c[i][j] = 1.0 
					end
				end
			}

		}

		# sum
		cwsum = 0.0
		wsum = 0.0
		n.times{|i|
			i.times{|j|
				cwsum += w[i][j] * c[i][j]
				wsum += w[i][j]
			}
		}

		return cwsum/wsum
	end
end

