
class StatisticsXY
	def initialize(x, y)
		raise(TypeError, "x should be an array") unless x.kind_of? Array
		raise(TypeError, "y should be an array") unless y.kind_of? Array
		raise("x and y should have the same size") unless x.size == y.size

		@x = x
		@y = y
	end

	def size
		return @x.size
	end

	#=======================================================================
	# Correlation measures
	#=======================================================================
	
	def pearson_r
		# means
		mean_y = 0.0
		mean_x = 0.0
		@x.each_index{|i|
			mean_y += @y[i] / @y.size
			mean_x += @x[i] / @y.size
		}
		# variances
		variance_y = 0.0
		variance_x = 0.0
		@x.each_index{|i|
			variance_y += (@y[i] - mean_y)**2 / @y.size
			variance_x += (@x[i] - mean_x)**2 / @y.size
		}
		stdev_y = variance_y**0.5
		stdev_x = variance_x**0.5
		# covariance
		covariance = 0.0
		@x.each_index{|i|
			i.times{|j|
				covariance += (@y[i] - @y[j] ) * (@x[i] - @x[j]) / @y.size**2
			}
		}
		# R
		r = covariance / (stdev_y * stdev_x)
		return r

	end

	def pearson_r_squared
		return pearson_r**2
	end
	
	def kendall_tau
		# https://en.wikipedia.org/wiki/Kendall_rank_correlation_coefficient
		pair_product = 0.0
		@x.each_index{|i|
			i.times{|j|
				pair_product += sgn(@x[i] - @x[j]) * sgn(@y[i] - @y[j])
			}
		}
		return pair_product * 2 / size / (size - 1)
	end

	def predictive_index
		#: Calculation of Predictive index
		#: as defined in https://pubs.acs.org/doi/full/10.1021/jm0100279

		# get w and c
		w = []
		c = []
		size.times{|i|
			w[i] = []
			c[i] = []
			i.times{|j|
				w[i][j] = (@x[j] - @x[i]).abs
				c[i][j] = 0.0
				if @y[j] != @y[i]
					if (@x[j]-@x[i])/(@y[j]-@y[i]) < 0.0
						c[i][j] = -1.0 
					end
					if (@x[j]-@x[i])/(@y[j]-@y[i]) > 0.0
						c[i][j] = 1.0 
					end
				end
			}

		}

		# sum
		cwsum = 0.0
		wsum = 0.0
		size.times{|i|
			i.times{|j|
				cwsum += w[i][j] * c[i][j]
				wsum += w[i][j]
			}
		}

		return cwsum/wsum
	end

	#=======================================================================
	# Helper methods
	#=======================================================================
	
	def sgn(x)
		return  0.0 if x == 0.0
		return -1.0 if x < 0.0
		return  1.0 if x > 0.0
		raise "Wrong argument #{x} passed to sgn(x)"
	end
end

