require "classes/geometry/geometry.rb"

module MolecularCharge
	def MolecularCharge.from_file(filename)
		unless File.exist?(filename)
			Cuby::error("Reading charge from file - file does not exist\n(#{filename})")
		end

		case filename
		when /.sdf$/
			# Charge from data fields
			charges = []
			geo = Geometry.from_file(filename)
			geo.info[:sdf_data].each_pair{|key, value|
				if key.downcase =~ /molecular_charge/
					charges << value.to_i
				end
				if key.downcase.strip  == "charge"
					charges << value.to_i
				end
			}
			charges.uniq!

			# No charge in data fields
			if charges.size == 0
				# Charge from CHG record
				if geo.info[:sdf_total_charge]
					Cuby::warning("Reading charge from SDF file - not found in data fields,\ncharge is computed from atom charges")
					return geo.info[:sdf_total_charge]
				end
				Cuby::warning("Reading charge from SDF file - no charge information found,i\nASSUMING NEUTRAL MOLECULE\n(no entry matching 'charge' or /molecular_charge/ found,\nand no charges on atoms read)")
				return 0
			end
			if charges.size > 1
				Cuby::error("Reading charge from SDF file - conflicting values found in data fields")
			end
			return charges.first
		when /.xyz$/
			geo = Geometry.from_file(filename)
			info = geo.get_extended_info
			if info.charge
				return info.charge
			else
				Cuby::error("Reading charge from .xyz file - charge record not found in the comment")
			end
		when /.pdb$/
			geo = Geometry.from_file(filename)
			info = geo.get_extended_info
			if info.charge
				return info.charge
			else
				Cuby::error("Reading charge from .pdb file - charge record not found in remarks")
			end
		when /.mol2$/
			geo = Geometry.from_file(filename)
			if geo.info[:geofile_charge]
				return geo.info[:geofile_charge]
			else
				Cuby::error("Reading charge from .mol2 file - charge record not found")
			end
		when /.txt$/
			charge = IO.readlines(filename).first.strip
			if charge =~ /^-*[0-9]+$/
				return charge.to_i
			else
				Cuby::error("Reading charge from .txt file - the file should contain a single number")
			end
		else
			Cuby::error("Reading charge from file - unsupported file format")
		end

	end
end
