require "classes/optimization/optimizer.rb"

#===============================================================================
# Nelder-Mead simplex optimization - gradient-free
#===============================================================================

class OptimizerGenetic < Optimizer

	class Gene
		attr_accessor :vector
		attr_accessor :fitness

		def initialize(vector)
			@vector = vector.deep_copy
			@fitness = nil
		end

		def mutate_all!(amount = 0.01)
			@vector.each_index{|index|
				change = (rand - 0.5) * 2.0 * amount
				@vector[index] = @vector[index] + change
			}
		end

		def mutate!(amount = 0.01)
			# Which variable to change
			index = rand(@vector.size)
			change = (rand - 0.5) * 2.0 * amount
			@vector[index] = @vector[index] + change
		end

		def crossover!(other_gene)
			index = rand(@vector.size)
			@vector[index] = other_gene.vector[index]
		end
	end

	class GeneArray < Array
	end

	def initialize(settings)
		# Generic initialization
		super(settings)

		@n = 10

		# Groups in the sorted population
		@g_keep = (0..0)
		@g_mutate = (1..3)
		@g_crossover = (4..6)
		@g_mutate_best = (7..9)

		@genes = GeneArray.new(@n)

	end

	def parallel
		return @n
	end

	def starting_vector=(vec)
		# Setter for @vector, to be used only upon initialization of the optimizer
		@vector = vec
		@n.times{|i|
			gene = Gene.new(@vector)
			if i > 0
				gene.mutate_all!(0.5)
			end
			@genes[i] = gene

		}
	end

	def hessian_estimate_default
		# Do nothing...
	end
	
	def initialize_hessian(argument)
		# Do nothing...
	end

	def what_to_calculate
		# By default, optimizer needs energy and gradient
		# Here, only energy is needed
		return [:energy]
	end
	
	def step
		# Queue calculations
		@genes.each_index{|i|
			@calculation_queue_proc.call(i,@genes[i].vector)
		}
		# Run calculations
		@genes.each_index{|i|
			@genes[i].fitness = @calculation_run_proc.call(i)[0]
		}
		# Sort genes
		@genes.sort!{|a,b| a.fitness <=> b.fitness }
		@energy = @genes.first.fitness
		@vector = @genes.first.vector

		@print_step_proc.call

		# Crossover with the better ones
		@g_crossover.each{|i|
			@genes[i].crossover!(@genes[rand(@g_mutate)])
		}
		# Goodones will be mutated
		@g_mutate.each{|i|
			@genes[i].mutate!
		}
		# make others be the mutation of the best
		@g_mutate_best.each{|i|
			@genes[i].vector = @genes[0].vector.deep_copy
			@genes[i].mutate!
		}
		return :continue
	end
	
end
