#!/usr/bin/env ruby
$:.unshift(File.dirname(`which cuby4`)).uniq! # Magic line to add path to the executable to $RUBYLIB
require "yaml"
require "protocols/dataset/classes/data_set"
require "classes/compatibility/ruby_backports.rb"

dataset_path = File.dirname(`which cuby4`)+"/data/datasets"

filename = ARGV[0].downcase + ".yaml"

f = File.open(dataset_path+"/"+filename)
set = YAML.unsafe_load(f)
f.close

class String
	def mdformat
		return self.gsub(/__([^_]+)__/, '<strong>\1</strong>').gsub(/_([^_]+)_/, '<em>\1</em>')
	end
end

# Basic info
puts "<h1>Data set #{set.description.name}</h1>"
# Set description
puts "<p><strong>#{set.description.text}</strong></p>"
# Data set size
puts "<p>"
s = "The data set contains #{set.items.size} items"
if set.plots
	s += " plottable as #{set.plots.size} curves"
end
s += "."
puts s
puts "</p>"

# Downloads
geodb = set.items.first.geometry.gsub(/:.*/,'')
puts "<h2>Downloads</h2>"
puts "<p>"
puts "Here you can download the <a href=\"download_datasets/#{filename}\">data set definition file</a> (in YAML format), an <a href=\"download_geometries/#{geodb}.tar\"> archive of the geometries</a> and a <a href=\"download_geometries/#{geodb}.yaml\">dictionary file</a> assigning the system names to the geometry file."
puts "</p>"

if set.description.geoms_with_metadata
  puts "<p>"
  puts "The geometry files for this data set contain metadata including the reference values."
  puts "</p>"
end

# Geometries
geometries = []
set.items.each{|item|}

# References
citations = {}
if set.description.references
	puts "<h2>References</h2>"
	puts "<ol>"
	set.description.references.each_pair{|doi, ref|
		puts "<li>"
		puts "<a href=\"http://dx.doi.org/#{doi}\">#{ref.mdformat}</a>"
		puts "</li>"
		citations[doi.strip] = citations.size + 1
	}
	puts "</ol>"
end

# Geometries
if set.description.method_geometry
	puts "<h2>Geometries</h2>"
	puts "<p>"
	puts set.description.method_geometry.split("\n").join("<br>\n").mdformat
	puts "</p>"
end

# Energies
if set.description.method_energy
	puts "<h2>Reference energies</h2>"
	puts "<p>"
	puts set.description.method_energy.split("\n").join("<br>\n").mdformat
	puts "</p>"
end

# Notes
if set.description.notes
	puts "<h2>Notes</h2>"
	puts "<p>"
	puts set.description.notes.split("\n").join("<br>\n").mdformat
	puts "</p>"
end

# Groups
if set.description.groups
	puts "<h2>Groups</h2>"
	puts "<p>"
	by = ""
	if set.description.groups_by
		by = " (#{set.description.groups_by})"
	end
	puts "The data set is divided into following groups#{by}:"
	puts "</p>"
	puts "<ul>"
	set.description.groups.each{|group|
		puts "<li>#{group}</li>"
	}
	puts "</ul>"
end

# Alternative references
if set.alternative_reference
	puts "<h2>Alternative reference data</h2>"
	puts "<p>"
	puts "The data set contains following sets of alternative reference values:"
	puts "</p>"
	puts "<ul>"
	set.alternative_reference.each_pair{|name, value|
		puts "<li>"
		puts name
		if set.best_reference && set.best_reference == name
			puts " (this is considered to be the most accuratere ference for this data set)"
		end
		if value.class == ProtocolDataset::DataSetReference
			if value.references
				reflist = []
				value.references.each_pair{|doi, cit|
					if citations[doi.strip]
						reflist << "<a href=\"http://dx.doi.org/#{doi}\" title=\"#{cit}\">[#{citations[doi.strip]}]</a>"
					else
						reflist << "<a href=\"http://dx.doi.org/#{doi}\" title=\"#{cit}\">[#{cit}]</a>"

					end
				}
				puts " " + reflist.join(",")
			end
		end
		puts "</li>"
	}
	puts "</ul>"
end
