#=======================================================================
# Prmtop reader/writer
#=======================================================================

# It is now used only to get charges from the prmtop file, but can be used
# to change them and some other properties

class AmberPrmtop
	#: Container for prmtop file. Reads some properties from the top, converting them
	#: to cuby units. These values can be modified and written back to the file.

	attr_reader :natom, :charges, :radii, :screens, :dihedral_fc, :atomtypes
	attr_writer :charges, :radii, :screens, :dihedral_fc

	def initialize(filename)
		read_top(filename)
	end

	def read_top(topfilename)
		#: Read data from prmtop, filing @natom, @charges, @radii and @screens
		@toplines = IO.readlines(topfilename)

		# get pointers
		@pointers = []
		i = 0
		while @toplines[i] !~ /FLAG POINTERS/
			i += 1
		end
		i += 2
		while @toplines[i] !~ /%/
			items = @toplines[i].split
			items.each { |item| @pointers.push(item.to_i)}
			i += 1
		end
		@natom = @pointers[0]

		#get charges
		@charges = []
		while @toplines[i] !~ /FLAG CHARGE/
			i += 1
		end
		i += 2
		while @toplines[i] !~ /%/
			items = @toplines[i].split
			items.each { |item| @charges << ((item.to_f / 18.2223 * 100000000).round.to_f / 100000000)}
			i += 1
		end

		# get dihedral force constants
		@dihedral_fc = []
		while @toplines[i] !~ /FLAG DIHEDRAL_FORCE_CONSTANT/
			i += 1
		end
		i += 2
		while @toplines[i] !~ /%/
			items = @toplines[i].split
			items.each { |item| @dihedral_fc << item.to_f }
			i += 1
		end

		# get atom types
		@atomtypes = []
		while @toplines[i] !~ /FLAG AMBER_ATOM_TYPE/
			i += 1
		end
		i += 2
		while @toplines[i] !~ /%/
			items = @toplines[i].split
			items.each { |item| @atomtypes << item }
			i += 1
		end

		# get radii
		@radii = []
		while @toplines[i] !~ /FLAG RADII/
			i += 1
		end
		i += 2
		while @toplines[i] !~ /%/
			items = @toplines[i].split
			items.each { |item| @radii << item.to_f }
			i += 1
		end

		# get screens
		@screens = []
		while @toplines[i] !~ /FLAG SCREEN/
			i += 1
		end
		i += 2
		while (@toplines[i] !~ /%/) && (i < @toplines.length)
			items = @toplines[i].split
			items.each { |item| @screens << item.to_f }
			i += 1
		end
	end

	def write_top(topfilename)
		#: Write prmtop, overwriting the respective sections with data from
		#: @charges, @radii and @screens
		file = File.new(topfilename,"w+")
		i = 0
		while @toplines[i] !~ /FLAG CHARGE/
			file.puts @toplines[i]
			i += 1
		end
		file.puts @toplines[i]
		i += 1
		file.puts @toplines[i]
		i += 1
		#now on first line, write out new charges
		@charges.each_index { |chi|
			file.printf("%16.8E",@charges[chi] * 18.2223)
			if (chi+1)%5 == 0
				file.printf("\n")
			else
				file.printf("\n") if chi == @charges.length - 1
			end
		}
		#seek charges end
		while @toplines[i] !~ /%/
			i += 1
		end

		# seek dihedral fc start
		while @toplines[i] !~ /DIHEDRAL_FORCE_CONSTANT/
			file.puts @toplines[i]
			i += 1
		end
		file.puts @toplines[i]
		i += 1
		file.puts @toplines[i]
		i += 1
		#now on first line, write out new fc
		@dihedral_fc.each_index { |fci|
			file.printf("%16.8E",@dihedral_fc[fci])
			if (fci+1)%5 == 0
				file.printf("\n")
			else
				file.printf("\n") if fci == @dihedral_fc.length - 1
			end
		}
		# seek dihedral fc end
		while @toplines[i] !~ /%/
			i += 1
		end

		# seek radii start
		while @toplines[i] !~ /FLAG RADII/
			file.puts @toplines[i]
			i += 1
		end
		file.puts @toplines[i]
		i += 1
		file.puts @toplines[i]
		i += 1
		#now on first line, write out new radii
		@radii.each_index { |chi|
			file.printf("%16.8E",@radii[chi])
			if (chi+1)%5 == 0
				file.printf("\n")
			else
				file.printf("\n") if chi == @radii.length - 1
			end
		}
		#seek radii end
		while @toplines[i] !~ /%/
			i += 1
		end
		
		# seek screen start
		while @toplines[i] !~ /FLAG SCREEN/
			file.puts @toplines[i]
			i += 1
		end
		file.puts @toplines[i]
		i += 1
		file.puts @toplines[i]
		i += 1
		#now on first line, write out new screen
		@screens.each_index { |chi|
			file.printf("%16.8E",@screens[chi])
			if (chi+1)%5 == 0
				file.printf("\n")
			else
				file.printf("\n") if chi == @screens.length - 1
			end
		}
		#seek screen end
		while (@toplines[i] !~ /%/) && (i < @toplines.length)
			i += 1
		end
		
		#continue with old top
		while i < @toplines.length
			file.puts @toplines[i]
			i += 1
		end
		file.close
	end
end
