module InterfaceCuby
	#=======================================================================
	# Interface header
	#=======================================================================
	# Interface status
	DEVELOPMENT_FLAG = :ok
	DEVELOPMENT_STATUS = ""
	# Interface information
	INTERFACE = :calculation_external
	CAPABILITIES = [:energy, :gradient, :ghost_atoms]
	MODIFIER = false
	DIRECTORY = "CUBY"
	METHODS = {
		:proxy => [:energy, :gradient, :ghost_atoms]
	}
	#=======================================================================
	
	CUBY_INP = "cuby_input.yaml"
	CUBY_OUT = "cuby_output.yaml"


	def prepare_interface
		# Prepare calculation directory
		if calc_dir_mkdir(CUBY_INP) == :old
			calc_using_input
			return
		end

		cuby_write_input

		# Save info on the system upon writing the input
		calc_writing_input
	end

	def calculate_interface
		unless @settings[:existing_calc_dir] == :read_results && 
			(FileTest.exist?(in_calc_dir(CUBY_OUT)) || FileTest.exist?(in_calc_dir("RESULTS/"+CUBY_OUT)))
			cuby_write_input # To refresh the geometry
			cuby_run_calculation 
		end
		results = cuby_read_results
		return results
	end

	def cleanup_interface
		calc_dir_delete if @settings[:job_cleanup]
	end

	#=======================================================================
	# Private methods
	#=======================================================================
	
	def cuby_write_input
		# Make a copy of the settings, copy without overwrite
		calc_settings = @settings.blocks[:calculation]
		# Save geometry in the settings
		calc_settings[:geometry] = @geometry.to_s
		# Copy charge and multiplicity
		calc_settings[:charge] = @settings[:charge]
		calc_settings[:multiplicity] = @settings[:multiplicity]

		# What to calculate
		calc_settings[:job] = :energy
		calc_settings[:job] = :gradient if @what.include?(:gradient)
		if (@what - [:energy, :gradient]).size > 0
			Cuby::error "Cuby interface now supports only energy and gradient"
		end

		# Ask for serialized results
		calc_settings[:write_results_yaml] = CUBY_OUT

		# Write settings as input file
		File.open(in_calc_dir(CUBY_INP), "w+") {|f|
			f.puts(calc_settings.to_hash.to_yaml)
		}
	end

	def cuby_run_calculation
		command = "cd #{calc_dir};"
		command << " cuby4 #{CUBY_INP} 2> cuby.err > cuby.log"
		system(command)
	end

	def cuby_read_results
		if FileTest.exist?(in_calc_dir(CUBY_OUT))
			fn = in_calc_dir(CUBY_OUT)
		elsif FileTest.exist?(in_calc_dir("RESULTS/"+CUBY_OUT))
			fn = in_calc_dir("RESULTS/"+CUBY_OUT)
		else
			Cuby::error "Output from external cuby calculation (#{CUBY_OUT}) not found in\n#{calc_dir}"
		end

		results = Results.from_yaml_file(fn)
		return results
	end

end
