# H-H repulsion for semiempirical methods

class D3HHFix
	def initialize(settings)
		# H-H fix for SQM

		# Store parameters
		@hh_k =  settings[:d3_hh_para][:k]
		@hh_e =  settings[:d3_hh_para][:e]
		@hh_r =  settings[:d3_hh_para][:r0]

		# Version 2
		if settings[:d3_hh_fix_version] == 2
			require "interfaces/dispersion3/classes/soft_core_repulsion"
			@hh_fix_2 = SoftCoreRepulsion.new(
				1.0, 1.5, 
				settings[:d3_hh_para][:k],
				settings[:d3_hh_para][:e],
				settings[:d3_hh_para][:e2])
		end

		if settings[:d3_hh_fix_version] == 3
			require "interfaces/dispersion3/classes/soft_core_repulsion"
			@hh_fix_2 = SoftCoreRepulsion.new(
				1.0, 1.3, 
				settings[:d3_hh_para][:k],
				settings[:d3_hh_para][:e],
				settings[:d3_hh_para][:e2])
			@hh_fix_cc = SoftCoreRepulsion.new(
				1.9, 2.2, 
				30000.0,
				-3.5
			)

		end
	end

	def calc_add_to_results(geometry, results)
		# Add the repulsion to the energy and gradient

		# Repulsion energy
		erep = 0.0
		# Calculate gradient?
		grad = results.gradient != nil

		geometry.each_index { |i|
			atom_a = geometry[i]
			next unless atom_a.element == :H
			(i).times { |j|
				atom_b = geometry[j]
				next unless atom_b.element == :H

				next if (atom_a.properties[:ghost] || atom_b.properties[:ghost] || 
					 atom_a.properties[:dummy] || atom_b.properties[:dummy])

				# Pairwise data
				r = atom_a.distance(atom_b)

				if @hh_fix_2
					erep += @hh_fix_2.calculate(r)
					deriv = @hh_fix_2.deriv(r) if grad
				else
					# energy
					erep += @hh_k * (1.0 - 1.0/(1.0 + Math::exp(-@hh_e*(r/@hh_r - 1.0))))
					# gradient
					if grad
						deriv = (1.0 / (1.0 + Math::exp(-@hh_e*(r/@hh_r-1.0)))**2 * @hh_e/@hh_r * Math::exp(-@hh_e*(r/@hh_r-1.0))) * -@hh_k
					end
				end

				# Add derivatives to the total gradient
				if grad
					# Project gradient size to coordinate
					dcrd= geometry[i] - geometry[j]
					dgr = deriv/r * dcrd

					# and apply to gradient
					results.gradient[i] += dgr
					results.gradient[j] -= dgr
				end
			}
		}

#		# C-C
#		geometry.each_index { |i|
#			atom_a = geometry[i]
#			next unless atom_a.element == :C
#			(i).times { |j|
#				atom_b = geometry[j]
#				next unless atom_b.element == :C
#
#				next if (atom_a.properties[:ghost] || atom_b.properties[:ghost] || 
#					 atom_a.properties[:dummy] || atom_b.properties[:dummy])
#
#				# Pairwise data
#				r = atom_a.distance(atom_b)
#
#				if @hh_fix_cc
#					erep += @hh_fix_cc.calculate(r)
#					deriv = @hh_fix_cc.deriv(r) if grad
#				end
#
#				# Add derivatives to the total gradient
#				if grad
#					# Project gradient size to coordinate
#					dcrd= geometry[i] - geometry[j]
#					dgr = deriv/r * dcrd
#
#					# and apply to gradient
#					results.gradient[i] += dgr
#					results.gradient[j] -= dgr
#				end
#			}
#		}
#
#		# C-O
#		geometry.each_index { |i|
#			atom_a = geometry[i]
#			(i).times { |j|
#				atom_b = geometry[j]
#				next unless (atom_a.element == :C && atom_b.element == :O) || (atom_a.element == :O && atom_b.element == :C)
#
#				next if (atom_a.properties[:ghost] || atom_b.properties[:ghost] || 
#					 atom_a.properties[:dummy] || atom_b.properties[:dummy])
#
#				# Pairwise data
#				r = atom_a.distance(atom_b)
#
#				if @hh_fix_cc
#					erep += @hh_fix_cc.calculate(r) / 2
#					deriv = @hh_fix_cc.deriv(r) / 2 if grad
#				end
#
#				# Add derivatives to the total gradient
#				if grad
#					# Project gradient size to coordinate
#					dcrd= geometry[i] - geometry[j]
#					dgr = deriv/r * dcrd
#
#					# and apply to gradient
#					results.gradient[i] += dgr
#					results.gradient[j] -= dgr
#				end
#			}
#		}

		# Save the repulsion energy as energy component
		results.energy_components[:hh_repulsion] = erep
		# Add it to the total energy
		results.energy += erep
	
		return nil	
	end

end
