import sys
try:
    import torch
except ImportError:
    print("---conda activate torchmd-net")
else:
    from torchmdnet.models.model import load_model

def data():
    el_to_z={'Br':1, 'C':3,  'Cl':7, 'F':9, 'H':10, 'I':12, 'N':17, 'O':21, 'P':23, 'S':26}
    cont=sys.stdin.readlines()
    model_file = cont[0].strip()
    cont=cont[3:]
    elem=[]
    for line,data in enumerate(cont):
        cont[line]=[float(i) for i in data.split()[1:]]
        elem.append(el_to_z[data.split()[0]])
    return (elem, cont, model_file)

def energy_forces(elem, geom, model_f):
    average=0
    model = load_model(model_f, derivative=True)
    types = torch.tensor(elem, dtype=torch.long)
    pos = torch.tensor(geom, dtype=torch.float32)
    energy, forces = model.forward(types, pos)#,batch)
    return (energy.item(), forces)

elements, coords, model_file = data()

ene, forces = energy_forces(elements, coords, model_file)

print(ene/4.184)
for f in forces:
    # Multiply with -1 to get gradient from forces
    x = f[0].item() / 4.184 * -1.0
    y = f[1].item() / 4.184 * -1.0
    z = f[2].item() / 4.184 * -1.0
    print("%25.15f%25.15f%25.15f" % (x,y,z))
