
module ProtocolDipole
	#=======================================================================
	# Protocol header
	#=======================================================================
	PROTOCOL_TYPE = :elementary_calculation
	#=======================================================================

	def print_header_protocol
		# Print header
		Cuby.log.print_cuby_logo(:comment => false, :text => 'Dipole calculation')
	end

	def prepare_protocol
		# Read geometry
		load_geometry
		# Create calculation
		@calculation = Calculation.new(@name, @settings, @geometry)
		# Prepare calculation
		what = [:energy]
		@calculation.prepare(what)
	end

	def run_protocol(queue)
		# Submit calculation
		@calculation.send_to_queue(queue)
	end

	def print_protocol
		# Print results
		results.print_dipole
	end

	def cleanup_protocol
		@calculation.cleanup
	end

	def results
		# Results are collected only first time they are needed
		unless @results
			# Get results
			@results = @calculation.results
		end
		return @results
	end

	#=======================================================================
	# Optional protocol functionality
	#=======================================================================
	
	def single_line_result
		return sprintf("Dipole size: %20.6f Debye", results.multipoles[:dipole].size * CUBY2DEBYE)
	end

	def single_number_result
		return results.multipoles[:dipole].size * CUBY2DEBYE
	end

	def update_geometry(geometry)
		# Update geometry, invalidate the results on previous one
		@geometry.copy_coordinates_from(geometry)
		@results = nil
	end

end
