require "classes/internal_coordinates/z_matrix.rb"

module ActionsZmat
	def build_zmat
		# No geometry is written
		@write_geo = false

		# Build connectivity
		connectivity = Connectivity.new(@geometry, @settings[:connectivity_tolerance])

		# Z-matrix 
		#zmat = ZMatrix.from_geometry_keep_order(@geometry)
		if @settings[:geometry_zmat_bondlist].size > 0
			bondlist = []
			@settings[:geometry_zmat_bondlist].each{|bond_sel|
				atomlist = @geometry.atomlist_from_selection(bond_sel)
				if atomlist.size != 2
					Cuby::error "Bond selection '#{bond_sel}' should select two atoms"
				end
				bondlist << atomlist
			}
			zmat = ZMatrix.from_geometry(@geometry, bondlist)
		else
			zmat = ZMatrix.from_geometry(@geometry)
		end

		case @settings[:geometry_zmat_format]
		when :gaussian
			puts zmat.to_s
		when :gaussian_vars
			zmat.build_variables!
			zmat.label_intermolecular_variables!
			puts zmat.to_s
		when :mopac
			puts zmat.to_s_mopac(:label => :user_bond)
		end
		puts
	end

	def zmat_update
		# Check for presence of z-matrix
		unless @geometry.info[:z_matrix]
			Cuby::error "Action zmat_update applicable only to Z-matrices"
		end
		zmat = @geometry.info[:z_matrix]

		# Load second geometry
		geometry2 = Geometry.from_file(@settings[:geometry2])

		# Update z-matrix
		if @geometry.size == geometry2.size
			zmat.update_zmat!(geometry2)
		else
			zmat.update_zmat_dummy!(geometry2)
		end

		# Rounding - not used
		if false
			decimals = 4
			zmat.lines.each{|line|
				line.coordinates.each_index{|i|
					if i == 0
						line.coordinates[i].value = line.coordinates[i].value.round(decimals)
					else
						line.coordinates[i].value = (line.coordinates[i].value / Math::PI * 180.0).round(decimals) / 180.0 * Math::PI
					end
				}
			}
		end


		# Update also the cartesian representation
		newgeo = zmat.to_geometry
		@geometry.clear
		@geometry.append!(newgeo)
	end

	def displacement_in_zmat
		# Z-matrix for @geometry
		if @geometry.info[:z_matrix]
			zmat = @geometry.info[:z_matrix]
		else
			# Build Z-matrix for input geometry
			zmat = ZMatrix.from_geometry_keep_order(@geometry)
			zmat.build_variables!
			zmat.label_intermolecular_variables2!
		end

		# Read geometry2
		geo2 = Geometry.from_file(@settings[:geometry2])

		# Calculate displacement
		vec = geo2.to_vector - @geometry.to_vector
		vec = vec.normalize * 0.1
		geo2.update_from_vector!(@geometry.to_vector + vec)
		zmat2 = zmat.deep_copy
		zmat2.update_zmat_dummy!(geo2)


		# Express difference as z-matrix, store in zmat2
		zmat2.each_coord_with_index{|coord, i|
			# Construct the coordinate class
			c = coord.type.to_s
			c += "_im" if coord.name =~ /_im$/
			c = c.to_sym
			# Apply displacement if coordinate class matches the input
			if @settings[:zmat_displace_coords].include?(c)
				coord.value = coord.value - zmat.coord_by_index(i).value
			else
				coord.value = 0.0
			end
		}

		# Update the z-matrix
		x = @settings[:zmat_displace_amount]
		zmat.each_coord_with_index{|coord, i|
			coord.value += zmat2.coord_by_index(i).value * x
		}

		# Update also the cartesian representation
		newgeo = zmat.to_geometry
		@geometry.clear
		@geometry.append!(newgeo)
	end
end
