require "protocols/optimize/coordinate_transformation.rb"

class CoordinateTransformationNone < CoordinateTransformation
	def initialize(geometry, settings)
		Cuby.log.puts_debug "No coordinate transformation - optimization in complete cartesian space"
		@dimension = geometry.size * 3
		@settings = settings
	end

	def geometry_to_vec(geometry)
		return geometry.to_vector
	end

	def gradient_to_vec(gradient)
		return gradient.to_vector
	end

	def vec_to_geometry!(geometry, vector)
		geometry.update_from_vector!(vector)
		return vector
	end

	def geometry_to_geometry(geometry)
		return geometry
	end
	
	def transformed_gradient(gradient)
		return gradient
	end
	
	def hessian_to_matrix(hessian)
		return hessian
	end
	
	def initial_hessian(stat_counters = nil)
		# Here, the default is single_number
		unless @settings.set?(:hessian_estimate)
			@settings[:hessian_estimate] = :single_number
		end

		case @settings[:hessian_estimate]
		when :single_number
			# The simplest estimate: unit matrix * some number
			return @settings[:opt_diagonal_h0]
		when :from_file
			# Does not work with freezing atoms
			if @settings[:optimize_region] and @settings[:optimize_region] != "%all()"
				Cuby::error("Cartesian hessian does not work with :from_file and :optimize_region.")
			end
			# Load a hessian file
			Cuby::log.puts "Reading hessian from file #{@settings[:hessian_read]}"
			f = File.open(@settings[:hessian_read], "r")
			cart_hess = YAML.unsafe_load(f)
			f.close
			return hessian_to_matrix(cart_hess)
		when :from_file_shift
			# Does not work with freezing atoms
			if @settings[:optimize_region] and @settings[:optimize_region] != "%all()"
				Cuby::error("Cartesian hessian does not work with :from_file and :optimize_region.")
			end
			# Load a hessian file
			Cuby::log.puts "Reading hessian from file #{@settings[:hessian_read]}"
			f = File.open(@settings[:hessian_read], "r")
			cart_hess = YAML.unsafe_load(f)
			f.close
			
			# Make it non-singular with min_hess value
			int_hess = hessian_to_matrix(cart_hess)
			eig_sys = int_hess.eigensystem
			
			min_hess = 1.0
			min = min_hess
			eig_sys[1] = eig_sys[1].to_vector
			eig_sys[1].each{|l| min = l if l < min}
			if min < min_hess
				eig_sys[1].each_index{|i| eig_sys[1][i] += min_hess - min}
				return eig_sys[0] * Matrix.diagonal(eig_sys[1]) * eig_sys[0].inverse
			else
				return int_hess
			end
		else
			Cuby::error("The selected hessian estimate is not available in cartesian coordinates")
		end
	end
end
