require "protocols/scan/classes/scan_generator"

class ScanGeneratorInterpolateZm < ScanGenerator

	SETUP_KEYS = ["steps"]

	def initialize(settings, geometry)
		# Check if the geometry has z-matrix
		unless geometry.info[:z_matrix]
			Cuby::error "Scan in z-matrix coordinates requires input geometry in z-matrix format"
		end
		@z_matrix = geometry.info[:z_matrix]

		# Load and check 2nd geometry
		geometry2 = Geometry.from_file(settings[:geometry2])
		unless geometry2.info[:z_matrix]
			Cuby::error "Scan in z-matrix coordinates requires 2nd geometry in z-matrix format"
		end
		@z_matrix2 = geometry2.info[:z_matrix]


		# Check for keys
		check_setup_at_least_one(settings, SETUP_KEYS)
		@steps = settings[:scan_generator_setup]["steps"].to_i
		if @steps < 1
			Cuby::error "At leat one step must be made"
		end
	end

	def iterate(geometry)
		# Calculate differences
		differences = []
		@z_matrix.each_coord_with_index{|c1,i|
			c2 = @z_matrix2.coord_by_index(i)
			differences[i] = c2.value - c1.value
		}


		(0..@steps).each{|i|
			tempzmat  = @z_matrix.deep_copy

			@z_matrix.each_coord_with_index{|startcoord,j|
				tempzmat.coord_by_index(j).value = startcoord.value + differences[j] * i / @steps
			}

			tempzmat.update_geometry!(geometry)
			str = sprintf("scan: %d", i)
			yield geometry, str
		}
		return @steps + 1

	end

end
