require "test/unit"
require "yaml"
# Load cuby
$:.unshift(File.dirname(__FILE__)+"/../..").uniq! # Magic line to add path to the executable to $RUBYLIB
require "classes/cuby.rb" # Load Cuby classes and libraries
Cuby.load_framework

class TestCustomYaml < Test::Unit::TestCase

	def test_matrix
		x = Matrix.unit(3)
		s = x.to_yaml
		x2 = YAML.unsafe_load(s)
		assert_instance_of(Matrix, x2)
		assert_equal(x, x2)
	end

	def test_vector
		x = Vector[1,2,3,4,5]
		s = x.to_yaml
		x2 = YAML.unsafe_load(s)
		assert_instance_of(Vector, x2)
		assert_equal(x, x2)
	end

	def test_coordinate
		x = Coordinate[1,2,3]
		s = x.to_yaml
		x2 = YAML.unsafe_load(s)
		assert_instance_of(Coordinate, x2)
		assert_equal(x, x2)
	end

	def test_Atom
		x = Atom.new(:Au, 0.5, 0.3, 0.1)
		s = x.to_yaml
		x2 = YAML.unsafe_load(s)
		assert_instance_of(Atom, x2)
		assert_equal(x, x2)
		x.properties[:mass] = 0.123
		s = x.to_yaml
		x2 = YAML.unsafe_load(s)
		assert_instance_of(Atom, x2)
		assert_equal(x, x2)
	end

	def test_geometry
		x = Geometry.new
		x << Atom.new(:Au, 0.5, 0.3, 0.1)
		x << Atom.new(:He, 2.5, 0.3, 0.1)
		x << Atom.new(:I,  0.5, -0.333, 11.1)
		s = x.to_yaml
		x2 = YAML.unsafe_load(s)
		assert_instance_of(Geometry, x2)
		assert_equal(x, x2)
	end

	def test_hessian
		x = Hessian.unit(5)
		s = x.to_yaml
		x2 = YAML.unsafe_load(s)
		assert_instance_of(Hessian, x2)
		assert_equal(x, x2)
	end

	def test_point_charge
		x = PointCharge.new(0.0, 1.1, 2.2, -1.5)
		s = x.to_yaml
		x2 = YAML.unsafe_load(s)
		assert_instance_of(PointCharge, x2)
		assert_equal(x, x2)
	end

end
