#!/usr/bin/env python3

import sys

import numpy as np
import mpl_toolkits.mplot3d as a3
import matplotlib.colors as colors
import matplotlib.pyplot as plt
import pylab as pl
import scipy as sp
import pandas

def plot_surface(filename, title=""):
    # Get data
    table =  pandas.read_csv(filename, delimiter="\t", header = None)
    x1 = table.values[:,0]
    y1 = table.values[:,1]
    z1 = table.values[:,2]
    x2 = table.values[:,3]
    y2 = table.values[:,4]
    z2 = table.values[:,5]
    x3 = table.values[:,6]
    y3 = table.values[:,7]
    z3 = table.values[:,8]
    r = table.values[:,9]/255.0
    g = table.values[:,10]/255.0
    b = table.values[:,11]/255.0
    # Make plot
    ax = a3.Axes3D(pl.figure(figsize=(10,10)))
    ax.set_aspect('equal', adjustable='box')
    ax.set_axis_off()
    ax.view_init(azim=0, elev=-60)
    x1min = x1.min()
    x1max = x1.max()
    y1min = y1.min()
    y1max = y1.max()
    z1min = z1.min()
    z1max = z1.max()
    for i in range(x1.size):
        # Shading in z
        #rz = (z1[i] - z1min) / (z1max - z1min)
        #sz = 0.5 + rz*0.5
        # Shading in x
        rz = (x1[i] - x1min) / (x1max - x1min)
        sz = 0.5 + rz*0.5
        vtx = sp.array([[x1[i],y1[i],z1[i]],[x2[i],y2[i],z2[i]],[x3[i],y3[i],z3[i]]])
        tri = a3.art3d.Poly3DCollection([vtx])
        tri.set_color(colors.rgb2hex([r[i]*sz,g[i]*sz,b[i]*sz]))
        
        #tri.set_edgecolor('k')
        ax.add_collection3d(tri)
    ax.set_xlim3d(-3, 3)
    ax.set_ylim3d(-3, 3)
    ax.set_zlim3d(-3, 3)
    #ax.set_autoscale_on(True)
    pl.title = title
    pl.savefig(filename+'.png', bbox_inches='tight', dpi = 150)

print(sys.argv[1])
plot_surface(sys.argv[1])
